"""Fetch and download DAT files."""
from datoso.configuration.folder_helper import Folders
from datoso_plugin_internetarchive.fetch import fetch_helper
from datoso_plugin_internetarchive.ia import Archive
from datoso_seed_enhanced import __prefix__


def fetch() -> None:
    """Fetch and download DAT files."""
    folder_helper = Folders(seed=__prefix__)
    folder_helper.clean_dats()
    folder_helper.create_all()
    archive = Archive(dat_folder='DATs', item='Enhanced-ROMs-2025')
    fetch_helper(archive, folder_helper, __prefix__)



"""Rules for the Super Famicom Speed Hacks seed."""
from datoso_seed_enhanced.dats import EnhancedDat

rules = [
    {
        'name': '32X MD+ Dat',
        '_class': EnhancedDat,
        'seed': 'enhanced',
        'priority': 50,
        'rules': [
            {
                'key': 'name',
                'operator': 'contains',
                'value': '32X',
            },
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'MD+',
            },
        ],
    },
    {
        'name': '32X MSU-MD Dat',
        '_class': EnhancedDat,
        'seed': 'enhanced',
        'priority': 50,
        'rules': [
            {
                'key': 'name',
                'operator': 'contains',
                'value': '32X',
            },
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'MSU-MD',
            },
        ],
    },
    {
        'name': 'Sega CD Hacks Dat',
        '_class': EnhancedDat,
        'seed': 'enhanced',
        'priority': 50,
        'rules': [
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'Mega CD Hacks',
            },
        ],
    },
    {
        'name': 'Mega Drive Enhanced Colors Dat',
        '_class': EnhancedDat,
        'seed': 'enhanced',
        'priority': 50,
        'rules': [
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'Mega Drive',
            },
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'Enhanced Colors',
            },
        ],
    },
    {
        'name': 'Mega Drive MD+ Dat',
        '_class': EnhancedDat,
        'seed': 'enhanced',
        'priority': 50,
        'rules': [
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'Mega Drive',
            },
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'MD+',
            },
        ],
    },
    {
        'name': 'Mega Drive Mode 1 CD Dat',
        '_class': EnhancedDat,
        'seed': 'enhanced',
        'priority': 50,
        'rules': [
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'Mega Drive',
            },
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'Mode 1 CD',
            },
        ],
    },
    {
        'name': 'Mega Drive MSU-MD Dat',
        '_class': EnhancedDat,
        'seed': 'enhanced',
        'priority': 50,
        'rules': [
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'Mega Drive',
            },
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'MSU-MD',
            },
        ],
    },
    {
        'name': 'Super Famicom Enhanced Colors Dat',
        '_class': EnhancedDat,
        'seed': 'enhanced',
        'priority': 50,
        'rules': [
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'Enhanced Colors',
            },
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'Super Famicom',
            },
        ],
    },
    {
        'name': 'Super Famicom Speed Hacks Dat',
        '_class': EnhancedDat,
        'seed': 'enhanced',
        'priority': 0,
        'rules': [
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'Speed Hacks',
            },
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'Super Famicom',
            },
        ],
    },
    {
        'name': 'Super Famicom MSU1-SFC Dat',
        '_class': EnhancedDat,
        'seed': 'enhanced',
        'priority': 50,
        'rules': [
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'MSU1',
            },
            {
                'key': 'name',
                'operator': 'contains',
                'value': 'Super Famicom',
            },
        ],
    },
]


def get_rules() -> list:
    """Get the rules."""
    return rules



"""Actions for the base seed."""
from datoso_seed_enhanced.dats import EnhancedDat

actions = {
    '{dat_origin}': [
        {
            'action': 'LoadDatFile',
            '_class': EnhancedDat,
        },
        {
            'action': 'DeleteOld',
            'folder': '{dat_destination}',
        },
        {
            'action': 'Copy',
            'folder': '{dat_destination}',
        },
        {
            'action': 'SaveToDatabase',
        },
    ],
}

def get_actions() -> dict:
    """Get the actions dictionary."""
    return actions



"""Initialize seed variables."""
__all__ = ['__author__', '__description__', '__version__']
__version__ = '1.1.0'
__author__ = 'Lacides Miranda'
__description__ = 'Seed Plugin scaffold for Datoso'
__prefix__ = 'Enhanced'



"""DAT files for Enhanced."""
import re

from datoso.configuration import config
from datoso.repositories.dat_file import XMLDatFile


class EnhancedDat(XMLDatFile):
    """Enhanced Dat class."""

    seed: str = 'enhanced'

    def initial_parse(self) -> list:
        # pylint: disable=R0801
        """Parse the dat file."""
        name = self.name

        name_array = name.split(' - ')
        match len(name_array):
            case 1:
                company = None
                system = name_array[0]
            case 2:
                company, system = name_array
            case 3 | 4 | 5:
                company, system, *suffix = name_array
                self.suffix = suffix
        self.company = company
        self.system = system
        self.overrides()

        if self.modifier or self.system_type:
            self.prefix = config.get('PREFIXES', self.modifier or self.system_type, fallback='')
        else:
            self.prefix = None

        return [self.prefix, self.company, self.system, self.suffix, self.get_date()]


    def get_date(self) -> str:
        """Get the date from the dat file."""
        if self.file:
            result = re.findall(r'\(.*?\)', str(self.file))
            self.date = result[len(result)-1][1:-1]
        return self.date
